<?php

/*************************************************************
 * Inject - dependencies are injected into classes
 * @link      http://limbercode.com
 *************************************************************/
class inject {

	protected static $instance;
	protected $resolved = [];
	protected $bindings = [];
	protected $instances = [];
	protected $aliases = [];
	protected $extenders = [];
	protected $buildStack = [];
	protected $globalResolvingCallbacks = [];
	protected $globalAfterResolvingCallbacks = [];
	protected $resolvingCallbacks = [];
	protected $afterResolvingCallbacks = [];
	public $contextual = [];

	/**
	 * Resolve the given type from the container
	 * @param  string $abstract
	 * @param  array $parameters
	 * @return mixed
	 */
	public function create($abstract,array $parameters = []) {

		$abstract = $this->getAlias($this->normalize($abstract));

		if(isset($this->instances[$abstract])) {
			return $this->instances[$abstract];
		}

		$concrete = $this->getConcrete($abstract);

		if($this->isBuildable($concrete,$abstract)) {
			$object = $this->build($concrete,$parameters);
		}
		else {
			$object = $this->create($concrete,$parameters);
		}

		foreach($this->getExtenders($abstract) as $extender) {
			$object = $extender($object,$this);
		}

		if($this->isShared($abstract)) {
			$this->instances[$abstract] = $object;
		}

		$this->fireResolvingCallbacks($abstract,$object);
		$this->resolved[$abstract] = true;

		return $object;
	}

	/**
	 * Get the concrete type for a given abstract
	 * @param  string $abstract
	 * @return mixed   $concrete
	 */
	protected function getConcrete($abstract) {

		if(!is_null($concrete = $this->getContextualConcrete($abstract))) {
			return $concrete;
		}

		if(!isset($this->bindings[$abstract])) {
			return $abstract;
		}

		return $this->bindings[$abstract]['concrete'];
	}

	/**
	 * Get the alias for an abstract if available
	 * @param  string $abstract
	 * @return string
	 */
	protected function getAlias($abstract) {

		if(!isset($this->aliases[$abstract])) {
			return $abstract;
		}

		return $this->getAlias($this->aliases[$abstract]);
	}

	/**
	 * Normalize the given class name by removing leading slashes
	 * @param  mixed $service
	 * @return mixed
	 */
	protected function normalize($service) {

		return is_string($service) ? ltrim($service,'\\') : $service;
	}

	/**
	 * Get the contextual concrete binding for the given abstract
	 * @param  string $abstract
	 * @return string|null
	 */
	protected function getContextualConcrete($abstract) {

		if(isset($this->contextual[end($this->buildStack)][$abstract])) {
			return $this->contextual[end($this->buildStack)][$abstract];
		}
	}

	/**
	 * Determine if the given concrete is buildable
	 * @param  mixed $concrete
	 * @param  string $abstract
	 * @return bool
	 */
	protected function isBuildable($concrete,$abstract) {

		return $concrete===$abstract || $concrete instanceof Closure;
	}

	/**
	 * Instantiate a concrete instance of the given type
	 * @param  string $concrete
	 * @param  array $parameters
	 * @return mixed
	 */
	public function build($concrete,array $parameters = []) {

		if($concrete instanceof Closure) {
			return $concrete($this,$parameters);
		}

		$reflector = new ReflectionClass($concrete);

		if(!$reflector->isInstantiable()) {
			$notif = basic::load("notif","core");

			if(!empty($this->buildStack)) {
				$previous = implode(', ',$this->buildStack);

				$notif->genNotifByNum($file = "core",$notifNum = 87100,$externalVar = array(
					$concrete,
					$previous
				));
			}
			else {
				$notif->genNotifByNum($file = "core",$notifNum = 87110,$externalVar = $concrete);
			}
		}

		$this->buildStack[] = $concrete;

		$constructor = $reflector->getConstructor();

		if(is_null($constructor)) {
			array_pop($this->buildStack);

			return new $concrete;
		}

		$dependencies = $constructor->getParameters();
		$parameters = $this->keyParametersByArgument($dependencies,$parameters);
		$instances = $this->getDependencies($dependencies,$parameters);
		array_pop($this->buildStack);

		return $reflector->newInstanceArgs($instances);
	}

	/**
	 * If extra parameters are passed by numeric ID, rekey them by argument name
	 * @param  array $dependencies
	 * @param  array $parameters
	 * @return array
	 */
	protected function keyParametersByArgument(array $dependencies,array $parameters) {

		foreach($parameters as $key => $value) {
			if(is_numeric($key)) {
				unset($parameters[$key]);

				$parameters[$dependencies[$key]->name] = $value;
			}
		}

		return $parameters;
	}

	/**
	 * Resolve all of the dependencies from the ReflectionParameters
	 * @param  array $parameters
	 * @param  array $primitives
	 * @return array
	 */
	protected function getDependencies(array $parameters,array $primitives = []) {

		$dependencies = [];

		foreach($parameters as $parameter) {
			$dependency = $parameter->getClass();

			if(array_key_exists($parameter->name,$primitives)) {
				$dependencies[] = $primitives[$parameter->name];
			}
			elseif(is_null($dependency)) {
				$dependencies[] = $this->resolveNonClass($parameter);
			}
			else {
				$dependencies[] = $this->resolveClass($parameter);
			}
		}

		return $dependencies;
	}

	/**
	 * Resolve a non-class hinted dependency
	 * @param  \ReflectionParameter $parameter
	 * @return mixed
	 */
	protected function resolveNonClass(ReflectionParameter $parameter) {

		if(!is_null($concrete = $this->getContextualConcrete('$'.$parameter->name))) {
			if($concrete instanceof Closure) {
				return call_user_func($concrete,$this);
			}
			else {
				return $concrete;
			}
		}

		if($parameter->isDefaultValueAvailable()) {
			return $parameter->getDefaultValue();
		}

		$notif = basic::load("notif","core");
		$notif->genNotifByNum($file = "core",$notifNum = 87120,$externalVar = array(
			$parameter,
			$parameter->getDeclaringClass()->getName()
		));
	}

	/**
	 * Resolve a class based dependency from the container
	 * @param  \ReflectionParameter $parameter
	 * @return mixed
	 * @throws Exception
	 */
	protected function resolveClass(ReflectionParameter $parameter) {

		try {
			return $this->create($parameter->getClass()->name);
		}

		catch(Exception $e) {
			if($parameter->isOptional()) {
				return $parameter->getDefaultValue();
			}

			throw $e;
		}
	}

	/**
	 * Get the extender callbacks for a given type
	 * @param  string $abstract
	 * @return array
	 */
	protected function getExtenders($abstract) {

		if(isset($this->extenders[$abstract])) {
			return $this->extenders[$abstract];
		}

		return [];
	}

	/**
	 * Determine if a given type is shared
	 * @param  string $abstract
	 * @return bool
	 */
	public function isShared($abstract) {

		$abstract = $this->normalize($abstract);

		if(isset($this->instances[$abstract])) {
			return true;
		}

		if(!isset($this->bindings[$abstract]['shared'])) {
			return false;
		}

		return $this->bindings[$abstract]['shared']===true;
	}

	/**
	 * Fire all of the resolving callbacks
	 * @param  string $abstract
	 * @param  mixed $object
	 * @return void
	 */
	protected function fireResolvingCallbacks($abstract,$object) {

		$this->shootCallbackArray($object,$this->globalResolvingCallbacks);

		$this->shootCallbackArray($object,$this->getCallbacksForType($abstract,$object,$this->resolvingCallbacks));

		$this->shootCallbackArray($object,$this->globalAfterResolvingCallbacks);

		$this->shootCallbackArray($object,$this->getCallbacksForType($abstract,$object,$this->afterResolvingCallbacks));
	}

	/**
	 * Fire an array of callbacks with an object
	 * @param  mixed $object
	 * @param  array $callbacks
	 * @return void
	 */
	protected function shootCallbackArray($object,array $callbacks) {

		foreach($callbacks as $callback) {
			$callback($object,$this);
		}
	}

	/**
	 * Get all callbacks for a given type
	 * @param  string $abstract
	 * @param  object $object
	 * @param  array $callbacksPerType
	 * @return array
	 */
	protected function getCallbacksForType($abstract,$object,array $callbacksPerType) {

		$results = [];

		foreach($callbacksPerType as $type => $callbacks) {
			if($type===$abstract || $object instanceof $type) {
				$results = array_merge($results,$callbacks);
			}
		}

		return $results;
	}
}
