<?php

/*************************************************************
 * Language class
 * - all functions about multi languages and link url's
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class lang {

	public static $currentLanguage;
	private static $cfg;

	/**
	 * Constructor for Lang
	 */
	public function __construct() {

		self::$cfg = basic::loadCfg("langConfig",false);
	}

	/**
	 * Detect languages from browser - will detect the user languages
	 * @param bool|false $statisticInfo - if true will return statistic info about the languages, if not, only the languages will be returned as array
	 * @return array
	 */
	public static function detectLanguagesFromBrowser($statisticInfo = false) {

		$langs = array();

		if(isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
			preg_match_all('/([a-z]{1,8}(-[a-z]{1,8})?)\s*(;\s*q\s*=\s*(1|0\.[0-9]+))?/i',$_SERVER['HTTP_ACCEPT_LANGUAGE'],$langParse);
		}

		if(isset($langParse[1]) && count($langParse[1])) {
			$langs = array_combine($langParse[1],$langParse[4]);

			foreach($langs as $lang => $value) {
				if($value==='') {
					$langs[$lang] = 1;
				}
			}
			arsort($langs,SORT_NUMERIC);
		}

		if($statisticInfo!==false) {
			return $langs;
		}

		return array_keys($langs);
	}

	/**
	 * Url maker/creator - Create's a full/half url for any given directory.
	 * The pro of this function is that it return's also the language that we use on the site at the specific time we use it.
	 * @param string (required) $directory - the directory to make an url
	 * @param bool|false (optional) $noDomain - decides if we want to include the domain of the site
	 * @return string - the url, if multi language and language is set then the result url will contain it's language
	 * @example lang::url($directory="blog");
	 */
	public static function url($directory,$noDomain = false) {

		$directory = lcfirst($directory);
		$langToPut = self::getCurrentLanguageForUrls();

		if(!$directory) {
			$langToPut = rtrim($langToPut,"/");
		}

		if($langToPut=="xx-xx" || $langToPut=="xx-xx/") {       // solving misdirect on the specific xx-xx demo lang
			//	$langToPut="";
		}

		if(self::$cfg['lang']['route']=="inPath") {
			if(router::isLocalhost()) {
				return router::$localHostNextDir."/".$langToPut.$directory;
			}
			else {
				return "/".$langToPut.$directory;
			}
		}
		elseif(self::$cfg['lang']['route']=="inDomain") {
			return "/".$directory;
		}
		else {
			return router::$protocol.self::getCurrentLanguageForUrls().router::$domain."/".$directory;
		}
	}

	/**
	 * Get current language for links - of the current site language state
	 * @param string|null $language - set certain language in url
	 * @return the current language link
	 * @example lang::getCurrentLanguageForUrls();
	 */
	public static function getCurrentLanguageForUrls($language = "") {

		if(self::$cfg['lang']['setBy']=="url") {
			$putLanguage = lang::$currentLanguage;
			if($language) {
				$putLanguage = $language;
			}
			if($putLanguage!=self::getDefaultLanguage()) {
				$breaker = "";
				if($putLanguage) {
					if(self::$cfg['lang']['route']=="inPath") {
						$breaker = "/";

						return $putLanguage.$breaker;
					}
					elseif(self::$cfg['lang']['route']=="inDomain") {
						$breaker = ".";

						return $putLanguage.$breaker;
					}
				}
				//	return false;
				//	return $putLanguage.$breaker;
			}    // gives the language code to use inside links	// if it's the same we don't have to change it in the links
		}
	}

	/**
	 * Get current language - of the current site
	 * @param bool|false $returnDescription - will return also the language description in an array
	 * @return string/array - the current language
	 * @example lang::getCurrentLanguage();
	 */
	public static function getCurrentLanguage($returnDescription = false) {

		if(!self::$currentLanguage) {
			self::$currentLanguage = self::getDefaultLanguage();

			if(isset($_COOKIE['language']) && self::$cfg['lang']['setBy']=="cookie") {    // get language from cookie
				if(array_key_exists($_COOKIE['language'],self::$cfg['lang']['available'])) {        // check if language is valid
					self::$currentLanguage = strtolower($_COOKIE['language']);
				}
			}
			elseif(self::$cfg['lang']['setBy']=="url") {
				self::getLanguageFromUrl();
			}
		}    // only if we don't have this already set

		if($returnDescription!==false) {
			if(isset(self::$cfg['lang']['default'][self::$currentLanguage])) {
				return array(self::$currentLanguage => self::$cfg['lang']['default'][self::$currentLanguage]);
			}
			elseif(isset(self::$cfg['lang']['available'][self::$currentLanguage])) {
				return array(self::$currentLanguage => self::$cfg['lang']['available'][self::$currentLanguage]);
			}
		}
		else {
			return self::$currentLanguage;
		}
	}

	/**
	 * Set current language - of the current site
	 * @param string $language - the language we want to set
	 * @example lang::setCurrentLanguage($language="jp");
	 */
	public static function setCurrentLanguage($language) {

		$language = strtolower(substr($language,0,strlen($language)));        // must be 2/5 chars language code
		if(array_key_exists($language,self::$cfg['lang']['default']) || array_key_exists($language,self::$cfg['lang']['available'])) {  // check if language can be set
			self::$currentLanguage = $language;
		}
		else {
			$notif = basic::load("notif","core");
			$notif->genNotifByNum($file = "core",$num = 1830,$externalVar = $language);
		}
	}

	/**
	 * Get default language - of the current site
	 * @param bool|false $returnDescription - will return also the language description in an array
	 * @return string/array - the default language
	 * @example lang::getDefaultLanguage();
	 */
	public static function getDefaultLanguage($returnDescription = false) {

		if(isset(self::$cfg['lang']['default'])) {
			if($returnDescription!==false) {
				return reset(self::$cfg['lang']['default']);
			}
			else {
				reset(self::$cfg['lang']['default']);

				return key(self::$cfg['lang']['default']);
			}
		}
		else {      // (fallBack)
			self::$cfg['lang']['default'] = array("en" => "english");
			if($returnDescription!==false) {
				return "english";
			}
			else {
				return "en";
			}
		}
	}

	/**
	 * Get language from url - of the current site
	 * @param string $path
	 * @return array|mixed
	 * @example lang::getLanguageFromUrl();
	 */
	public static function getLanguageFromUrl($path = "") {

		if(!router::$protocol) {
			if(!empty($_SERVER['REQUEST_SCHEME'])) {
				$serverProtocol = $_SERVER['REQUEST_SCHEME'];
			}
			else {
				$serverProtocol = isset($_SERVER["HTTPS"]) ? "https" : "http";
			}

			$uri = $_SERVER['REQUEST_URI'];

			if(router::isLocalhost()) {
				$uri = str_replace(router::$localHostNextDir,"",$uri);
			}

			$fullAddress = $serverProtocol."://".$_SERVER['SERVER_NAME'].$uri;
			//	if((strlen($uri)>=2 && strlen($uri)<=6) || (router::isLocalhost() && !strlen($uri))) {
			//	if((strlen($uri)>=2 && strlen($uri)<=5) || (router::isLocalhost() && !strlen($uri))) {

			$isFromAvailableLanguages = array_key_exists(ltrim($uri,"/"),lang::getAvailableLanguages());

			if($isFromAvailableLanguages || (router::isLocalhost() && !strlen($uri))) {
				$fullAddress = $fullAddress."/";
			}

			$matches = array();                                    // initialize the matches

			if(self::$cfg['lang']['route']=="inPath") {
				//	$regexLang = '/((?:http[s]*\:\/\/)*.*?\/+)([a-z\-]{2,5})([\/\?]+)([^\?]*)/';
				//	$regexLang = '/((?:http[s]*\:\/\/)*)(.*)?\/+([a-z\-]{2,5})([\/\?]+)([^\?]*)/';
				//	$regexLang = '/((?:http[s]*\:\/\/)*)([^\/]*)?\/+([a-z\-]{2,5})([\/\?]+)([^\?]*)/';
				$regexLang = '/((?:http[s]*\:\/\/)*)([^\/]*)?\/?([a-z\-]{2,5})?([\/])([^\?]*)/';

				// tests case:
				// http://limbercode.dev/he/
				// http://limbercode.dev/he
				// http://limbercode.dev/he/documentation/classes/lang
				// http://limbercode.dev/he/HE/WW/DD/FF/documentation/overview - should fail 404
				// http://limbercode.dev/hedocumentation - should fail 404
				// http://limbercode.dev/hee/documentation - should fail 404
				// http://limbercode.dev/he2222/documentation - should fail 404

				if(preg_match($regexLang,$fullAddress,$matches)) {        // if language is requested
					router::$protocol = $matches[1];
					router::$domain   = $matches[2];
					router::$breaker  = $matches[4];
					router::$path     = $matches[5];

					if(router::isLocalhost()) {
						router::$domain = rtrim(router::$localHost,"/");
					}

					if(isset($matches[3])) {
						if(array_key_exists(strtolower($matches[3]),lang::getAvailableLanguages()) || $matches[3]==lang::getDefaultLanguage()) {    // check if language is valid
							self::$currentLanguage = $matches[3];

							if(isset($matches[5])) {
								router::$path = $matches[5];
							}
						}
						elseif(isset($matches[5]) && $matches[3]) {
							router::$path = $matches[3]."/".$matches[5];
						}
					}
				}
				else {
					router::$path = ltrim($_SERVER['REQUEST_URI'],"/");
				}
			}
			elseif(self::$cfg['lang']['route']=="inDomain") {
				//	$regexLang = '/((?:http[s]*\:\/\/)*)(.*?)(\.)(?=[^\/]*\..{2,5})+([^\?]*)/';
				$regexLang = '/((?:http[s]*\:\/\/)*)(.*?)(\.)(?=[^\/]*\..{2,5})+([^\/?]*)?([^\?]*)/';

				// tests case:
				// www.mydomain.com/whatever/my-site            // matches: www
				// mydomain.com                                 // does not match
				// http://mydomain.com                          // does not match
				// https://mydomain.com                         // does not match
				// banana.com/somethingelse                     // does not match
				// https://banana.com/somethingelse.org         // does not match
				// http://what-ever.mydomain.mu                 // matches: what-ever
				// dev-www.thisdomain.com/whatever              // matches: dev-www
				// hot-MamaSitas.SomE_doma-in.au.xxx            // matches: hot-MamaSitas
				// http://hot-MamaSitas.SomE_doma-in.au.xxx     // matches: hot-MamaSitas
				// пуст.пустыня.ru                              // even non english chars! Woohoo! matches: пуст
				// пустыня.ru                                   // does not match

				// tests case:
				// http://he.en.limbercode.dev/he/HE/WW/DD/FF/documentation/overview - should fail 404
				// http://he.limbercode.dev/hedocumentation - should fail 404
				// http://he.limbercode.dev/documentation
				// http://he.limbercode.dev/
				// http://he.limbercode.dev

				if(preg_match($regexLang,$fullAddress,$matches)) {        // if language is requested
					router::$protocol = $matches[1];
					router::$domain   = $matches[4];
					router::$breaker  = $matches[3];
					router::$path     = $matches[5];

					if(array_key_exists(strtolower($matches[2]),lang::getAvailableLanguages()) || $matches[2]==lang::getDefaultLanguage()) {    // check if language is valid
						$restOfUrl = preg_split($regexLang,$fullAddress);

						self::$currentLanguage = $matches[2];
						if(isset($matches[4])) {
							$path = $matches[4];
						}
					}
				}
				else {
					router::disassembleUrl();
				}
			}

			router::extractSubDomain();

			if($path) {
				return array(self::$currentLanguage,router::$path);
			}
		}

		return self::$currentLanguage;
	}

	/**
	 * Get available languages - of the current site
	 * @param bool|false $withoutCurrent
	 * @return the available languages
	 * @example lang::getAvailableLanguages();
	 */
	public static function getAvailableLanguages($withoutCurrent = false) {

		if(!isset(self::$cfg['lang']['default'])) {
			self::$cfg['lang']['default'] = array("en" => "english");   // fallback to english
		}

		if(!isset(self::$cfg['lang']['available'])) {
			//	return self::getDefaultLanguage();
			return self::$cfg['lang']['default'];
		}

		if($withoutCurrent!==false) {
			$all = self::$cfg['lang']['default'] + self::$cfg['lang']['available'];
			unset($all[self::getCurrentLanguage()]);

			return $all;
		}
		else {
			return self::$cfg['lang']['default'] + self::$cfg['lang']['available'];
		}
	}

	/**
	 * Language url start - return's the start of the url in other language
	 * @param bool|string $language - the requested language
	 * @param bool|false $noDomain
	 * @return string - the url
	 * @example lang::languageUrl($language="he");
	 */
	public static function languageUrlStart($language = false,$noDomain = false) {

		if(!array_key_exists(strtolower($language),lang::getAvailableLanguages())) {    // check if language is valid
			if(router::isLocalhost()) {
				return rtrim(router::$localHostNextDir,"/");
			}
		}

		if(!$language) {
			$language = lang::getDefaultLanguage();
		}

		if(self::$cfg['lang']['route']=="inPath") {
			if($noDomain==true) {
				return self::getCurrentLanguageForUrls($language);
			}

			$languageToPut = self::getCurrentLanguageForUrls($language);
			$languageToPut = rtrim($languageToPut,'/');
			if($languageToPut) {
				$languageToPut = "/".$languageToPut;
			}

			return router::$protocol.router::$domain.$languageToPut;
		}
		elseif(self::$cfg['lang']['route']=="inDomain") {
			return router::$protocol.self::getCurrentLanguageForUrls($language).router::$domain;
		}
	}

	/**
	 * Language url - return's the current page url in other language
	 * @param string $language - the requested language
	 * @param bool|false $noDomain
	 * @return string - the url
	 * @example lang::languageUrl($language="he");
	 */
	public static function languageUrl($language,$noDomain = false) {

		if(self::$cfg['lang']['route']=="inPath") {
			if($noDomain==true) {
				return self::getCurrentLanguageForUrls($language).router::$path;
			}

			return router::$protocol.router::$domain."/".self::getCurrentLanguageForUrls($language).router::$path;
		}
		elseif(self::$cfg['lang']['route']=="inDomain") {
			$path = router::$path;
			if(!stringManipulation::startsWith(router::$path,"/")) {
				$path = "/".router::$path;
			}

			return router::$protocol.self::getCurrentLanguageForUrls($language).router::$domain.$path;
		}
	}

	/**
	 * Get current language for directories - if we are on default language, it will return false - so we can use main dir
	 * @return bool|string
	 * @example lang::getCurrentLanguage();
	 */
	public static function getCurrentLanguageForDir() {

		$languageDir = self::getCurrentLanguage();

		if(isset(self::$cfg['lang']['default'][self::$currentLanguage])) {
			return false;
		}

		return $languageDir.DS;
	}
}

