<?php

/*************************************************************
 * Memory usage and timing class
 * - memory usage information
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class memoryUsageAndTiming {

	private $break;
	private $realUsage;
	private $startTime;
	private $stopTime;
	private $statistics = array();

	/**
	 * Constructor for Memory Usage Information
	 * @param boolean $realUsage
	 * @param string $break
	 */
	public function __construct($realUsage = false,$break = "<br>") {

		$this->break = $break;
		$this->start();        // start timing
		$this->setStart();    // set start

		$this->realUsage = $realUsage;
	}

// Mem sector:

	/**
	 * Get current memory usage
	 * @param boolean $withStyle
	 * @return integer - returns current memory usage with or without styling
	 */
	public function getCurrentMemoryUsage($withStyle = true) {

		$mem = memory_get_usage($this->realUsage);

		return ($withStyle) ? $this->byteFormat($mem) : $mem;
	}

	/**
	 * Get peak memory usage
	 * @param boolean $withStyle
	 * @return integer - returns peak of memory usage
	 */
	public function getPeakMemoryUsage($withStyle = true) {

		$mem = memory_get_peak_usage($this->realUsage);

		return ($withStyle) ? $this->byteFormat($mem) : $mem;
	}

	/**
	 * Set memory usage - set memory usage with info
	 * @param string /null $info
	 */
	public function setMemoryUsage($info = "") {

		$this->statistics[] = array(
			'time'         => time(),
			'info'         => $info,
			'memory_usage' => $this->getCurrentMemoryUsage()
		);
	}

	/**
	 * Print memory usage information - print all memory usage info and memory limit and
	 */
	public function printMemoryUsageInformation() {

		foreach($this->statistics as $satistic) {
			//	echo  "Time: ".$satistic['time'].
			//			" | Memory Usage: ".$satistic['memory_usage'].
			//			" | Info: ".$satistic['info'];
			//	echo $this->break;
		}

		$timing = $this->printTotalExecutionTime();

		//	echo $this->break;	//	.$this->break;
		$memoryUsage = "Peak of memory usage: ".$this->getPeakMemoryUsage().", ";
		//	echo $this->break;$this->break;

		$theme = basic::$cfg['template']['theme']."Theme";
		if(method_exists($theme,"notif")) {
			$theme::notif($memoryUsage.$timing,$notifType = "efficiency");
		}
		else echo $memoryUsage.$timing;
	}

	/**
	 * Set start - set start with default info or some custom info
	 * @param string /null $info
	 */
	public function setStart($info = "Initial Memory Usage") {

		$this->setMemoryUsage($info);
	}

	/**
	 * Set end - set end with default info or some custom info
	 * @param string /null $info
	 */
	public function setEnd($info = "Memory Usage at the End") {

		$this->setMemoryUsage($info);
	}

	/**
	 * Byte format - byte formatting
	 * @param integer $bytes
	 * @param string /null $unit
	 * @param integer $decimals
	 * @return string/integer - the output
	 */
	private function byteFormat($bytes,$unit = "",$decimals = 2) {

		$units = array(
			'B'  => 0,
			'KB' => 1,
			'MB' => 2,
			'GB' => 3,
			'TB' => 4,
			'PB' => 5,
			'EB' => 6,
			'ZB' => 7,
			'YB' => 8
		);

		$value = 0;
		if($bytes>0) {
			// Generate automatic prefix by bytes
			// If wrong prefix given
			if(!array_key_exists($unit,$units)) {
				$pow  = floor(log($bytes) / log(1024));
				$unit = array_search($pow,$units);
			}

			// Calculate byte value by prefix
			$value = ($bytes / pow(1024,floor($units[$unit])));
		}

		// If decimals is not numeric or decimals is less than 0
		// then set default value
		if(!is_numeric($decimals) || $decimals<0) {
			$decimals = 3;
		}

		// Format output
		return sprintf('%.'.$decimals.'f '.$unit,$value);
	}

// Timing sector:

	/**
	 * Start - set start time
	 */
	public function start() {

		$this->startTime = microtime(true);
	}

	/**
	 * Stop - set stop/end time
	 */
	public function stop() {

		$this->stopTime = microtime(true);
	}

	/**
	 * Get elapsed time
	 * @return integer - returns time elapsed from start
	 */
	public function getElapsedTime() {

		return $this->getExecutionTime(microtime(true));
	}

	/**
	 * Get total execution time
	 * @return integer - returns total execution time
	 */
	public function getTotalExecutionTime() {

		if(!$this->stopTime) {
			return false;
		}

		return $this->getExecutionTime($this->stopTime);
	}

	/**
	 * Get full stats
	 * @return integer - returns start time, stop time and total execution time
	 */
	public function getFullStats() {

		if(!$this->stopTime) {
			return false;
		}

		$stats                         = array();
		$stats['startTime']            = $this->getDateTime($this->startTime);
		$stats['stopTime']             = $this->getDateTime($this->stopTime);
		$stats['total_execution_time'] = $this->getExecutionTime($this->stopTime);

		return $stats;
	}

	/**
	 * Print elapsed time - prints time elapsed from start
	 */
	public function printElapsedTime() {

		echo $this->break.$this->break;
		echo "Elapsed time: ".$this->getExecutionTime(microtime(true));
		echo $this->break.$this->break;
	}

	/**
	 * Print total execution time
	 */
	public function printTotalExecutionTime() {

		if(!$this->stopTime) {
			$this->stop();
			//	return false;
		}

		//	echo $this->break.$this->break;
		return "Total execution time: ".round($this->getExecutionTime($this->stopTime),3)." seconds";
		//	echo $this->break.$this->break;
	}

	/**
	 * Print full stats - prints start time, stop time and total execution time
	 */
	public function printFullStats() {

		if(!$this->stopTime) {
			return false;
		}

		echo $this->break.$this->break;
		echo "Script start date and time: ".$this->getDateTime($this->startTime);
		echo $this->break;
		echo "Script stop end date and time: ".$this->getDateTime($this->stopTime);
		echo $this->break.$this->break;
		echo "Total execution time: ".$this->getExecutionTime($this->stopTime);
		echo $this->break.$this->break;
	}

	/**
	 * Get date time
	 * @param string /null $time - unix time
	 * @return date - return format time to date and time
	 */
	private function getDateTime($time) {

		return date("Y-m-d H:i:s",$time);
	}

	/**
	 * Get execution time
	 * @param string /null $time - unix time
	 * @return date - return execution time by timestamp
	 */
	private function getExecutionTime($time) {

		return $time - $this->startTime;
	}
}

