<?php

/*************************************************************
 * User agent parser
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class userAgent {

	private static $cfg;

	/**
	 * Constructor for userAgent
	 */
	public function __construct() {

		self::$cfg = basic::loadCfg("userAgent",false);       // userAgent config is set to load privately
	}

	/**
	 * Parse user agent data
	 * @param string (optional) $userAgent - the user agent string
	 * @return array - the user agent information
	 */
	public static function parseUserAgent($userAgent = "") {

		if(!$userAgent) {
			$userAgent = "";    // just a reset. and yes, it is possible to be left out without "HTTP_USER_AGENT"
			if(isset($_SERVER['HTTP_USER_AGENT'])) {
				$userAgent = $_SERVER['HTTP_USER_AGENT'];
			}
		}
		$userAgent = trim(strtolower($userAgent));

		if(!$userAgent) {
			return false;
		}

		$bot             = self::parseBots($userAgent);
		$mobile          = "";
		$platformPlusVer = "";
		$browserPlusVer  = "";

		if(!$bot) {
			$mobile          = self::parseMobile($userAgent);
			$platformPlusVer = self::parsePlatformPlusVer($userAgent);
			$browserPlusVer  = self::parseBrowserPlusVer($userAgent);
		}

		return array(
			"bot"         => ($bot['bot']) ? $bot['bot'] : "",
			"mobile"      => (isset($mobile['mobile'])) ? $mobile['mobile'] : "",
			"platform"    => (isset($platformPlusVer['platformName'])) ? $platformPlusVer['platformName'] : "",
			"platformVer" => (isset($platformPlusVer['platformVer'])) ? $platformPlusVer['platformVer'] : "",
			"browser"     => (isset($browserPlusVer['browserName'])) ? $browserPlusVer['browserName'] : "",
			"browserVer"  => (isset($browserPlusVer['browserVer'])) ? $browserPlusVer['browserVer'] : "",
		);
	}

	/**
	 * Parse bots from user agent
	 * @param string (required) $userAgent - the user agent string
	 * @return array|bool
	 */
	public static function parseBots($userAgent) {

		foreach(self::$cfg['userAgent']['botNames'] as $key => $value) {        // first we will check for bots
			if(stripos($userAgent,$key)!==false) {
				return array('bot' => $value);
			}
		}

		return false;
	}

	/**
	 * Parse mobile from user agent
	 * @param string (required) $userAgent - the user agent string
	 * @return array|bool
	 */
	public static function parseMobile($userAgent) {

		foreach(self::$cfg['userAgent']['mobileNames'] as $key => $value) {        // first we will check for mobiles
			if(stripos($userAgent,$key)!==false) {
				return array('mobile' => $value);
			}
		}

		return false;
	}

	/**
	 * Parse browser from user agent
	 * @param string (required) $userAgent - the user agent string
	 * @return array|bool
	 */
	public static function parseBrowserPlusVer($userAgent) {

		$browser = array();

		foreach(self::$cfg['userAgent']['browserNames'] as $key => $value) {
			// Build regex that matches phrases for known browsers (e.g. "Firefox/2.0" or "MSIE 6.0").
			// This only matches the major and minor version numbers (e.g. "2.0.0.6" is parsed as simply "2.0").
			$pattern = '#'.$key.'[/ ]+([0-9]+(?:\.[0-9]+)?)#';

			if(preg_match($pattern,$userAgent,$matches)) {
				$browser['browserName'] = $value;

				if(isset($matches[1])) {
					$browser['browserVer'] = $matches[1];
				}

				return $browser;
			}
		}

		return false;
	}

	/**
	 * Parse platform from user agent
	 * @param string (required) $userAgent - the user agent string
	 * @return array|bool
	 */
	public static function parsePlatformPlusVer($userAgent) {

		$platform = array();
		foreach(self::$cfg['userAgent']['platformNames'] as $key => $value) {        // first we will check for platforms
			if(stripos($userAgent,$key)!==false) {
				$platform['platformName'] = $value;
				break;
			}
		}
		foreach(self::$cfg['userAgent']['platformVers'] as $key => $value) {        // first we will check for platforms vers
			if(stripos($userAgent,$key)!==false) {
				$platform['platformVer'] = $value;
				break;
			}
		}

		if(!empty($platform)) {
			return $platform;
		}

		return false;
	}
}

